@@ -20,7 +20,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: hash_table.c,v 1.2 2005/01/06 20:12:22 steinm Exp $
+ * $Id: hash_table.c,v 1.3 2007/07/04 13:07:59 steinm Exp $
  *
  ********************************************************************/
 
@@ -32,25 +32,31 @@
 
 #include "ght_hash_table.h"
 
+#ifdef WIN32
+#define PSLIB_INLINE
+#else
+#define PSLIB_INLINE inline
+#endif
+
 /* Flags for the elements. This is currently unused. */
 #define FLAGS_NONE     0 /* No flags */
 #define FLAGS_NORMAL   0 /* Normal item. All user-inserted stuff is normal */
 #define FLAGS_INTERNAL 1 /* The item is internal to the hash table */
 
 /* Prototypes */
-static inline void              transpose(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry);
-static inline void              move_to_front(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry);
-static inline void              free_entry_chain(ght_hash_table_t *p_ht, ght_hash_entry_t *p_entry);
-static inline ght_hash_entry_t *search_in_bucket(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_key_t *p_key, unsigned char i_heuristics);
+static PSLIB_INLINE void              transpose(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry);
+static PSLIB_INLINE void              move_to_front(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry);
+static PSLIB_INLINE void              free_entry_chain(ght_hash_table_t *p_ht, ght_hash_entry_t *p_entry);
+static PSLIB_INLINE ght_hash_entry_t *search_in_bucket(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_key_t *p_key, unsigned char i_heuristics);
 
-static inline void              hk_fill(ght_hash_key_t *p_hk, int i_size, void *p_key);
-static inline ght_hash_entry_t *he_create(ght_hash_table_t *p_ht, void *p_data, unsigned int i_key_size, void *p_key_data);
-static inline void              he_finalize(ght_hash_table_t *p_ht, ght_hash_entry_t *p_he);
+static PSLIB_INLINE void              hk_fill(ght_hash_key_t *p_hk, int i_size, void *p_key);
+static PSLIB_INLINE ght_hash_entry_t *he_create(ght_hash_table_t *p_ht, void *p_data, unsigned int i_key_size, void *p_key_data);
+static PSLIB_INLINE void              he_finalize(ght_hash_table_t *p_ht, ght_hash_entry_t *p_he);
 
 /* --- private methods --- */
 
 /* Move p_entry one up in its list. */
-static inline void transpose(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry)
+static PSLIB_INLINE void transpose(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry)
 {
   /*
    *  __    __    __    __
@@ -90,7 +96,7 @@ static inline void transpose(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_
 }
 
 /* Move p_entry first */
-static inline void move_to_front(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry)
+static PSLIB_INLINE void move_to_front(ght_hash_table_t *p_ht, ght_uint32_t l_bucket, ght_hash_entry_t *p_entry)
 {
   /*
    *  __    __    __
@@ -120,7 +126,7 @@ static inline void move_to_front(ght_hash_table_t *p_ht, ght_uint32_t l_bucket,
 }
 
 /* Search for an element in a bucket */
-static inline ght_hash_entry_t *search_in_bucket(ght_hash_table_t *p_ht, ght_uint32_t l_bucket,
+static PSLIB_INLINE ght_hash_entry_t *search_in_bucket(ght_hash_table_t *p_ht, ght_uint32_t l_bucket,
 						 ght_hash_key_t *p_key, unsigned char i_heuristics)
 {
   ght_hash_entry_t *p_e;
@@ -151,7 +157,7 @@ static inline ght_hash_entry_t *search_in_bucket(ght_hash_table_t *p_ht, ght_uin
 }
 
 /* Free a chain of entries (in a bucket) */
-static inline void free_entry_chain(ght_hash_table_t *p_ht, ght_hash_entry_t *p_entry)
+static PSLIB_INLINE void free_entry_chain(ght_hash_table_t *p_ht, ght_hash_entry_t *p_entry)
 {
   ght_hash_entry_t *p_e = p_entry;
 
@@ -165,7 +171,7 @@ static inline void free_entry_chain(ght_hash_table_t *p_ht, ght_hash_entry_t *p_
 
 
 /* Fill in the data to a existing hash key */
-static inline void hk_fill(ght_hash_key_t *p_hk, int i_size, void *p_key)
+static PSLIB_INLINE void hk_fill(ght_hash_key_t *p_hk, int i_size, void *p_key)
 {
   assert(p_hk);
 
@@ -174,7 +180,7 @@ static inline void hk_fill(ght_hash_key_t *p_hk, int i_size, void *p_key)
 }
 
 /* Create an hash entry */
-static inline ght_hash_entry_t *he_create(ght_hash_table_t *p_ht, void *p_data,
+static PSLIB_INLINE ght_hash_entry_t *he_create(ght_hash_table_t *p_ht, void *p_data,
 					  unsigned int i_key_size, void *p_key_data)
 {
   ght_hash_entry_t *p_he;
@@ -212,7 +218,7 @@ static inline ght_hash_entry_t *he_create(ght_hash_table_t *p_ht, void *p_data,
 }
 
 /* Finalize (free) a hash entry */
-static inline void he_finalize(ght_hash_table_t *p_ht, ght_hash_entry_t *p_he)
+static PSLIB_INLINE void he_finalize(ght_hash_table_t *p_ht, ght_hash_entry_t *p_he)
 {
   assert(p_he);
 
