@@ -14,12 +14,14 @@
  *   and will be supported by J. Hafner.
  *
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
-#include "config.h"
 #include "ps_intern.h"
 #include "ps_fontenc.h"
 #include "ps_error.h"
@@ -254,7 +256,7 @@ static float paramfloat() {
 static ADOBEINFO *newchar(PSDoc *psdoc) {
 	ADOBEINFO *ai ;
 
-	ai = (ADOBEINFO *) psdoc->malloc(psdoc, (unsigned long) sizeof(ADOBEINFO), "newchar: allocate memory for new characters") ;
+	ai = (ADOBEINFO *) psdoc->malloc(psdoc, (unsigned long) sizeof(ADOBEINFO), "newchar: allocate memory for new character") ;
 	ai->adobenum = -1 ;
 	ai->texnum = -1 ;
 	ai->width = -1 ;
@@ -321,10 +323,12 @@ static ADOBEINFO * handlechar(PSDoc *psdoc, ADOBEFONTMETRIC *metric) { /* an inp
 	ai = newchar(psdoc) ;
 	ai->adobenum = paramnum() ;
 	if(!expect(";")) {
+		psdoc->free(psdoc, ai);
 		ps_error(psdoc, PS_RuntimeError, _("Expected ';' in afm file."));
 		return(NULL);
 	}
 	if(!expect("WX")) {
+		psdoc->free(psdoc, ai);
 		ps_error(psdoc, PS_RuntimeError, _("Expected 'WX' in afm file."));
 		return(NULL);
 	}
@@ -333,15 +337,19 @@ static ADOBEINFO * handlechar(PSDoc *psdoc, ADOBEFONTMETRIC *metric) { /* an inp
 //		metric->adobeptrs[ai->adobenum] = ai ;
 //	}
 	if(!expect(";")) {
+		psdoc->free(psdoc, ai);
 		ps_error(psdoc, PS_RuntimeError, _("Expected ';' in afm file."));
 		return(NULL);
 	}
 	if(!expect("N")) {
+		psdoc->free(psdoc, ai);
 		ps_error(psdoc, PS_RuntimeError, _("Expected 'N' in afm file."));
 		return(NULL);
 	}
 	ai->adobename = paramnewstring(psdoc) ;
 	if(!expect(";")) {
+		psdoc->free(psdoc, ai->adobename);
+		psdoc->free(psdoc, ai);
 		ps_error(psdoc, PS_RuntimeError, _("Expected ';' in afm file."));
 		return(NULL);
 	}
@@ -427,6 +435,48 @@ int calculatekern(ADOBEINFO *ai, ADOBEINFO *succ) {
 }
 /* }}} */
 
+/* addkern() {{{
+ *
+ * Adds a new kerning pair to the list. Does not check if the pair
+ * already exists. The new pair is inserted at the beginning of the
+ * list.
+ */
+void addkern(PSDoc *psdoc, ADOBEINFO *ai, ADOBEINFO *succ, int kerning) {
+	KERN *nk ;
+
+	if((NULL == ai) || (NULL == succ)) {
+		error("One of the glyphs is not set.");
+		return;
+	}
+	nk = newkern(psdoc) ;
+	nk->succ = newstring(psdoc, succ->adobename);
+	nk->delta = kerning;
+	nk->next = ai->kerns ;
+	ai->kerns = nk ;
+}
+/* }}} */
+
+/* addligature() {{{
+ *
+ * Adds a new ligature to the list. Does not check if the ligature
+ * already exists. The new ligature is inserted at the beginning of the
+ * list.
+ */
+void addligature(PSDoc *psdoc, ADOBEINFO *ai, ADOBEINFO *succ, ADOBEINFO *sub) {
+	LIG *nl ;
+
+	if((NULL == ai) || (NULL == succ) || (NULL == sub)) {
+		error("One of the glyphs is not set.");
+		return;
+	}
+	nl = newlig(psdoc) ;
+	nl->succ = newstring(psdoc, succ->adobename);
+	nl->sub = newstring(psdoc, sub->adobename) ;
+	nl->next = ai->ligs ;
+	ai->ligs = nl ;
+}
+/* }}} */
+
 /* gfindadobe() {{{
  *
  * Finds metric of a glyph by its name.
@@ -670,7 +720,15 @@ case XHeight:
 			break ;
 case C: {
 			ADOBEINFO *hai = handlechar(psdoc, metric) ;
-			ght_insert(metric->gadobechars, hai, strlen(hai->adobename)+1, hai->adobename);
+			if(gfindadobe(metric->gadobechars, hai->adobename)) {
+				ps_error(psdoc, PS_Warning, _("Trying to insert the glyph '%s' which already exists. Please check your afm file for duplicate glyph names."), hai->adobename);
+				/* FIXME: Freeing the name and the ADOBEINFO structure may not be
+				 * enough. There can be kerning pairs and ligatures.
+				 */
+				psdoc->free(psdoc, hai->adobename);
+				psdoc->free(psdoc, hai);
+			} else
+				ght_insert(metric->gadobechars, hai, strlen(hai->adobename)+1, hai->adobename);
 			break ;
 }
 case KPX:
@@ -810,18 +868,18 @@ static void rmkern(PSDoc *psdoc, ght_hash_table_t *gadobechars, char *s1, char *
 }
 /* }}} */
 
-/* addkern() {{{
+/* copykern() {{{
  * Make the kerning for character S1 equivalent to that for S2.
  * If either S1 or S2 do not exist, do nothing.
  * If S1 already has kerning, do nothing.
  */
-static void addkern(PSDoc *psdoc, ght_hash_table_t *gadobechars, char *s1, char *s2)
+static void copykern(PSDoc *psdoc, ght_hash_table_t *gadobechars, char *s1, char *s2)
 {
 	ADOBEINFO *ai1 = gfindadobe(gadobechars, s1);
 	ADOBEINFO *ai2 = gfindadobe(gadobechars, s2);
 	if (ai1 && ai2 && !ai1->kerns) {
 		/* Put the new one at the head of the list, since order is immaterial.  */
-		ADOBEPTR *ap = (ADOBEPTR *) psdoc->malloc(psdoc, (unsigned long) sizeof(ADOBEPTR), "addkern: adobeptr");
+		ADOBEPTR *ap = (ADOBEPTR *) psdoc->malloc(psdoc, (unsigned long) sizeof(ADOBEPTR), "copykern: adobeptr");
 		ap->next = ai2->kern_equivs;
 		ap->ch = ai1;
 		ai2->kern_equivs = ap;
@@ -870,14 +928,14 @@ static void checkligkern(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, char *s)
 			}
 			if (n == 3 && strcmp(mlist[1], "{}") == 0) { /* rmkern command */
 				rmkern(psdoc, gadobechars, mlist[0], mlist[2], (ADOBEINFO *)0) ;
-			} else if (n == 3 && strcmp(mlist[1], "<>") == 0) { /* addkern */
+			} else if (n == 3 && strcmp(mlist[1], "<>") == 0) { /* copykern */
 				/* FIXME: What is the meaning of kern_equivs?
-				 * addkern() does not set the kern by kern_equivs which is not
+				 * copykern() does not set the kern by kern_equivs which is not
 				 * used.
 				 */
-				addkern(psdoc, gadobechars, mlist[0], mlist[2]) ;
+				copykern(psdoc, gadobechars, mlist[0], mlist[2]) ;
 			} else if (n == 3 && strcmp(mlist[0], "||") == 0 &&
-										strcmp(mlist[1], "=") == 0) { /* bc command */
+				strcmp(mlist[1], "=") == 0) { /* bc command */
 				ADOBEINFO *ai = gfindadobe(gadobechars, "||") ;
 
 				if (boundarychar != -1) {
@@ -921,10 +979,17 @@ static void checkligkern(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, char *s)
 						ps_error(psdoc, PS_RuntimeError, _("You can't lig to the boundary character!"));
 						return;
 					}
-//					if (gfindadobe(gadobechars, mlist[3]))	{
-//							ps_error(psdoc, PS_RuntimeError, _("Ligature not available in font."));
-//						return;
-//					}
+					/* FIXME: The following code is commented because the staticligkern
+					 * instruction didn't work anymore. For some reason e.g. the endash
+					 * wasn't found, though it was part of the font. I appears that
+					 * at this point the list of glyphs on gadobechars isn't complete
+					 */
+					/*
+					if (gfindadobe(gadobechars, mlist[3]))	{
+						ps_error(psdoc, PS_RuntimeError, _("Ligature '%s' not available in font."), mlist[3]);
+						return;
+					}
+					*/
 					if (!metrics->fixedpitch) { /* fixed pitch fonts get *0* ligs */
 						for (lig=ai->ligs; lig; lig = lig->next)
 							if (strcmp(lig->succ, mlist[1]) == 0)
@@ -936,6 +1001,8 @@ static void checkligkern(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, char *s)
 							lig->next = ai->ligs ;
 							ai->ligs = lig ;
 						}
+						if(lig->sub)
+							psdoc->free(psdoc, lig->sub);
 						lig->sub = newstring(psdoc, mlist[3]) ;
 						lig->op = op ;
 						if (strcmp(mlist[1], "||")==0) {
@@ -1107,23 +1174,26 @@ int readencoding(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, const char *enc) {
 		if (ignoreligkern == 0 && sawligkern == 0) {
 			getligkerndefaults(psdoc, metrics) ;
 		}
+		metrics->fontenc = ps_build_enc_hash(psdoc, e);
+		if(metrics->codingscheme)
+			psdoc->free(psdoc, metrics->codingscheme);
+		metrics->codingscheme = newstring(psdoc, e->name);
 		/* FIXME: For now we through away the fontenc from the file and
 		 * take the predefined font encoding.
 		 */
-/*		for (i=0; i<256; i++) {
+		for (i=0; i<256; i++) {
 			psdoc->free(psdoc, e->vec[i]);
 		}
 		psdoc->free(psdoc, e->name);
 		psdoc->free(psdoc, e);
-		e = &fontencoding ;*/
 	} else { /* not font encoding file name passed */
-		e = &fontencoding ;
+		e = &fontencoding[0] ;
 		getligkerndefaults(psdoc, metrics) ;
+		metrics->fontenc = ps_build_enc_hash(psdoc, e);
+		if(metrics->codingscheme)
+			psdoc->free(psdoc, metrics->codingscheme);
+		metrics->codingscheme = newstring(psdoc, e->name);
 	}
-	metrics->fontenc = ps_build_enc_hash(psdoc, e);
-	if(metrics->codingscheme)
-		psdoc->free(psdoc, metrics->codingscheme);
-	metrics->codingscheme = newstring(psdoc, e->name);
 	param = 0 ;
 	return 0 ;
 }
