@@ -1,6 +1,7 @@
+#include <string.h>
 #include "ps_intern.h"
 
-#define NUM_INPUTENCODINGS 2
+#define NUM_INPUTENCODINGS 4
 
 ENCODING inputencoding[NUM_INPUTENCODINGS] = {{
 	"ISO-8859-1",
@@ -25,7 +26,7 @@ ENCODING inputencoding[NUM_INPUTENCODINGS] = {{
 	 "space", "exclamdown", "cent", "sterling", "Euro", "yen", "brokenbar", "section",
 	 "dieresis", "copyright", "ordfeminine", "guillemotleft", "logicalnot", "hyphen", "registered", "macron",
 	 "degree", "plusminus", "twosuperior", "threesuperior", "acute", "mu", "paragraph", "periodcentered",
-	 "cedilla", "onesuperior", "ordmaskuline", "guillemotright", "onequarter", "onehalf", "threequaters", "questiondown",
+	 "cedilla", "onesuperior", "ordmasculine", "guillemotright", "onequarter", "onehalf", "threequaters", "questiondown",
 	 "Agrave", "Aacute", "Acircumflex", "Atilde", "Adieresis", "Aring", "AE", "Ccedilla",
 	 "Egrave", "Eacute", "Ecircumflex", "Edieresis", "Igrave", "Iacute", "Icircumflex", "Idieresis",
 	 "Eth", "Ntilde", "Ograve", "Oacute", "Ocircumflex", "Otilde", "Odieresis", "multiply",
@@ -66,13 +67,20 @@ ENCODING inputencoding[NUM_INPUTENCODINGS] = {{
 	 "racute", "aacute", "acircumflex", "abreve", "adieresis", "lacute", "cacute", "ccedilla",
 	 "ccaron", "eacute", "eogonek", "edieresis", "ecaron", "iacute", "icircumflex", "dcaron",
 	 "dcroat", "nacute", "ncaron", "oacute", "ocircumflex", "ohungerumlaut", "odieresis", "divide",
-	 "rcaron", "uring", "uacute", "uhungarumlaut", "udieresis", "yacute", "tcommaaccent", "dotaccent" } } };
+	 "rcaron", "uring", "uacute", "uhungarumlaut", "udieresis", "yacute", "tcommaaccent", "dotaccent" } },
+	 {NULL, NULL},
+	 {NULL, NULL}
+	};
 
 ENCODING *ps_get_inputencoding(const char *name) {
 	int i;
 
 	for(i=0; i<NUM_INPUTENCODINGS; i++) {
+#ifdef WIN32
+		if(0 == _stricmp(name, inputencoding[i].name)) {
+#else
 		if(0 == strcasecmp(name, inputencoding[i].name)) {
+#endif
 			return(&inputencoding[i]);
 		}
 	}
