@@ -40,6 +40,11 @@
 #include "hyphen.h"
 #endif
 
+#ifndef WIN32
+#define max(a,b) ((a)>(b) ? (a) : (b))
+#define min(a,b) ((a)<(b) ? (a) : (b))
+#endif
+
 #define PS_BORDER_UNSET  0
 #define PS_BORDER_SOLID  1
 #define PS_BORDER_DASHED 2
@@ -319,6 +324,8 @@ struct bookmark_ {
 #define PS_SCOPE_FONT     128
 #define PS_SCOPE_GLYPH    256
 
+#define LINEBUFLEN    1024
+
 struct PSDoc_ {
 	/* Document Information */
 	char  *Keywords;
@@ -432,6 +439,10 @@ struct PSDoc_ {
 };
 
 /* ps_util.c */
+void ps_write(PSDoc *p, const void *data, size_t size);
+void ps_putc(PSDoc *p, char c);
+void ps_puts(PSDoc *p, const char *s);
+void ps_printf(PSDoc *p, const char *fmt, ...);
 void* ps_ght_malloc(size_t size, void *data);
 void ps_ght_free(void *ptr, void *data);
 PS_RESOURCE **ps_get_resources(PSDoc *psdoc, const char *category, int *count);
@@ -451,7 +462,8 @@ ght_hash_table_t *ps_build_enc_from_font(PSDoc *psdoc, ADOBEFONTMETRIC *metrics)
 ENCODING *ps_build_enc_vector(PSDoc *psdoc, ght_hash_table_t *hashvec);
 void ps_list_fontenc(PSDoc *psdoc, ght_hash_table_t *hashvec);
 void ps_free_enc_vector(PSDoc *psdoc, ENCODING *enc);
-unsigned char ps_fontenc_code(ght_hash_table_t *fontenc, char *adobename);
+unsigned char ps_fontenc_code(PSDoc *psdoc, ght_hash_table_t *fontenc, char *adobename);
+int ps_fontenc_has_glyph(PSDoc *psdoc, ght_hash_table_t *fontenc, char *adobename);
 char *ps_inputenc_name(PSDoc *psdoc, unsigned char c);
 void ps_enter_scope(PSDoc *psdoc, int scope);
 void ps_leave_scope(PSDoc *psdoc, int scope);
@@ -463,6 +475,8 @@ int get_optlist_element_as_float(PSDoc *psdoc, ght_hash_table_t *opthash, const
 int get_optlist_element_as_int(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, int *value);
 int get_optlist_element_as_bool(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, ps_bool *value);
 int get_optlist_element_as_string(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, char **value);
+void ps_ascii85_encode(PSDoc *psdoc, char *data, size_t len);
+void ps_asciihex_encode(PSDoc *psdoc, char *data, size_t len);
 
 /* ps_afm.c */
 int calculatekern(ADOBEINFO *ai, ADOBEINFO *succ);
@@ -470,6 +484,8 @@ ADOBEFONTMETRIC *readadobe(PSDoc *psdoc, const char *filename);
 ADOBEINFO *gfindadobe(ght_hash_table_t *adobechars, const char *p);
 int readprotusion(PSDoc *psdoc, PSFont *psfont, const char *filename);
 int readencoding(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, const char *enc);
+void addkern(PSDoc *psdoc, ADOBEINFO *ai, ADOBEINFO *succ, int kerning);
+void addligature(PSDoc *psdoc, ADOBEINFO *ai, ADOBEINFO *succ, ADOBEINFO *sub);
 
 #include "ps_inputenc.h"
 #include "ps_fontenc.h"
