@@ -18,19 +18,71 @@
  *    Boston, MA 02111-1307, USA.
  */
 
-/* $Id: ps_util.c,v 1.39 2007/04/16 14:40:18 steinm Exp $ */
+/* $Id: ps_util.c,v 1.44 2007/07/11 16:03:57 steinm Exp $ */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
-#include "config.h"
 #include "ps_intern.h"
 #include "ps_memory.h"
 #include "ps_fontenc.h"
 #include "ps_inputenc.h"
 #include "ps_error.h"
 
+/* ps_write() {{{
+ * Output data using the current writeprocedure
+ */
+void
+ps_write(PSDoc *p, const void *data, size_t size) {
+	(void) p->writeproc(p, (void *) data, size);
+}
+/* }}} */
+
+/* ps_putc() {{{
+ * Output char
+ */
+void
+ps_putc(PSDoc *p, char c)
+{
+	ps_write(p, (void *) &c, (size_t) 1);
+}
+/* }}} */
+
+/* ps_puts() {{{
+ * Output string
+ */
+void
+ps_puts(PSDoc *p, const char *s)
+{
+	ps_write(p, (void *) s, strlen(s));
+}
+/* }}} */
+
+/* ps_printf() {{{
+ * Output formated string
+ */
+void
+ps_printf(PSDoc *p, const char *fmt, ...)
+{
+	char buf[LINEBUFLEN];  /* formatting buffer */
+	va_list ap;
+
+	va_start(ap, fmt);
+
+	setlocale(LC_NUMERIC, "C");
+	(void) vsnprintf(buf, LINEBUFLEN, fmt, ap);
+	setlocale(LC_NUMERIC, "");
+	ps_puts(p, buf);
+
+	va_end(ap);
+}
+/* }}} */
+
 /* ps_ght_malloc() {{{
  * 
  * malloc routine which is passed to the hash table functions.
@@ -403,7 +455,7 @@ ght_hash_table_t *ps_build_enc_hash(PSDoc *psdoc, ENCODING *enc) {
 		ght_set_alloc(hashvec, ps_ght_malloc, ps_ght_free, psdoc);
 		for(i=0; i<256; i++) {
 			if(strlen(enc->vec[i]) > 0)
-				ght_insert(hashvec, (void *) i+1, strlen(enc->vec[i])+1, enc->vec[i]);
+				ght_insert(hashvec, (char *) i+1, strlen(enc->vec[i])+1, enc->vec[i]);
 		}
 	}
 	return(hashvec);
@@ -415,7 +467,6 @@ ght_hash_table_t *ps_build_enc_hash(PSDoc *psdoc, ENCODING *enc) {
  * This will only include those glyphs which have a number > 0.
  */
 ght_hash_table_t *ps_build_enc_from_font(PSDoc *psdoc, ADOBEFONTMETRIC *metrics) {
-	int i;
 	ght_hash_table_t *hashvec;
 
 	hashvec = ght_create(512);
@@ -428,7 +479,7 @@ ght_hash_table_t *ps_build_enc_from_font(PSDoc *psdoc, ADOBEFONTMETRIC *metrics)
 		for(p_e = ght_first(metrics->gadobechars, &iterator, (void **) &p_key); p_e; p_e = ght_next(metrics->gadobechars, &iterator, (void **) &p_key)) {
 			/* Do not add glyphs with a number <= 0 */
 			if(p_e->adobenum > 0) {
-				if(0 > ght_insert(hashvec, (void *) p_e->adobenum+1, strlen(p_e->adobename)+1, p_e->adobename))
+				if(0 > ght_insert(hashvec, (char *) p_e->adobenum+1, strlen(p_e->adobename)+1, p_e->adobename))
 				fprintf(stderr, "Could not insert entry %d->%s into font encoding hash table.\n", p_e->adobenum, p_e->adobename);
 			}
 		}
@@ -505,13 +556,14 @@ void ps_free_enc_vector(PSDoc *psdoc, ENCODING *enc) {
 /* ps_fontenc_code() {{{
  * Returns the index in the fontencoding vector
  */
-unsigned char ps_fontenc_code(ght_hash_table_t *fontenc, char *adobename) {
+unsigned char ps_fontenc_code(PSDoc *psdoc, ght_hash_table_t *fontenc, char *adobename) {
 	if(fontenc) {
 		int code;
 		code = (int) ght_get(fontenc, strlen(adobename)+1, (void *) adobename);
 		if(code)
 			return((unsigned char) code -1);
 		else {
+			ps_error(psdoc, PS_Warning, _("The font encoding vector does not contain the glyph '%s'. Using '?' instead."), adobename);
 			return '?';
 		}
 	} else {
@@ -520,6 +572,24 @@ unsigned char ps_fontenc_code(ght_hash_table_t *fontenc, char *adobename) {
 }
 /* }}} */
 
+/* ps_fontenc_has_glpyh() {{{
+ * checks of a glyph is in the fontencoding vector
+ */
+int ps_fontenc_has_glyph(PSDoc *psdoc, ght_hash_table_t *fontenc, char *adobename) {
+	if(fontenc) {
+		int code;
+		code = (int) ght_get(fontenc, strlen(adobename)+1, (void *) adobename);
+		if(code)
+			return ps_true;
+		else {
+			return ps_false;
+		}
+	} else {
+		return ps_false;
+	}
+}
+/* }}} */
+
 /* ps_inputenc_name() {{{
  * Returns the adobe name for a char in the input encoding vector
  */
@@ -557,14 +627,18 @@ FILE *ps_open_file_in_path(PSDoc *psdoc, const char *filename) {
 	int i;
 	char buffer[255];
 
-	if(fp = fopen(filename, "r"))
+	if(fp = fopen(filename, "rb"))
 		return fp;
 
 	if(NULL != (pathres = ps_get_resources(psdoc, "SearchPath", &count))) {
 		for(i=count-1; i>=0; i--) {
+#ifdef HAVE_SNPRINTF
 			snprintf(buffer, 255, "%s/%s", pathres[i]->value, filename);
+#else
+			sprintf(buffer, "%s/%s", pathres[i]->value, filename);
+#endif
 			fprintf(stderr, "Searching for %s\n", buffer);
-			if(fp = fopen(buffer, "r")) {
+			if(fp = fopen(buffer, "rb")) {
 				fprintf(stderr, "found %s in %s\n", filename, pathres[i]->value);
 				break;
 			}
@@ -574,9 +648,15 @@ FILE *ps_open_file_in_path(PSDoc *psdoc, const char *filename) {
 			return(fp);
 	}
 
+#ifdef PACKAGE_DATA_DIR
+#ifdef HAVE_SNPRINTF
 	snprintf(buffer, 255, "%s/%s", PACKAGE_DATA_DIR, filename);
-	if(fp = fopen(buffer, "r"))
+#else
+	sprintf(buffer, "%s/%s", PACKAGE_DATA_DIR, filename);
+#endif
+	if(fp = fopen(buffer, "rb"))
 		return fp;
+#endif
 
 	return NULL;
 }
@@ -586,7 +666,7 @@ int pow85[5] = {1, 85, 85*85, 85*85*85, 85*85*85*85};
 	
 /* ps_ascii85_encode() {{{
  */
-int ps_ascii85_encode(PSDoc *psdoc, char *data, size_t len) {
+void ps_ascii85_encode(PSDoc *psdoc, char *data, size_t len) {
 	unsigned long buffer;
 	unsigned char c;
 	int i, count, cc;
@@ -636,7 +716,7 @@ int ps_ascii85_encode(PSDoc *psdoc, char *data, size_t len) {
 
 /* ps_asciihex_encode() {{{
  */
-int ps_asciihex_encode(PSDoc *psdoc, char *data, size_t len) {
+void ps_asciihex_encode(PSDoc *psdoc, char *data, size_t len) {
 	int i, cc=0;
 	unsigned char *dataptr;
 
@@ -709,7 +789,7 @@ ght_hash_table_t *ps_parse_optlist(PSDoc *psdoc, const char *optstr) {
 
 				/* Value and Name is read, insert it into hash */
 				if(name[0]) {
-					ght_insert(opthash, (void *) ps_strdup(psdoc, value), strlen(name)+1, name);
+					ght_insert(opthash, (char *) ps_strdup(psdoc, value), strlen(name)+1, name);
 					name[0] = '\0';
 					value[0] = '\0';
 				}
